--[[
    Aaron MacDonald
    December 3, 2010
]]

require('gfx')

require('data.particles')

bullets = {}

function bullets.create(sh)
    local effect = particle.createEffect(
      BULLET_PARTICLES[sh.type.bulletTrailName] )
    particle.start(effect)

    return { startX = sh.x,
             startY = sh.y,

             angle = sh.angle,

             dist = 0,
             maxDist = sh.type.bulletRange,
             speed = sh.type.bulletSpeed,

             sprite = gfx.createSprite(sh.type.bulletSpriteName),
             effect = effect }
end

function bullets.getCoords(b, areaWidth, areaHeight)
    local x, y = utility.polarToCartesian(b.angle, b.dist,
      b.startX, b.startY)

    x = utility.wrap(x, areaWidth)
    y = utility.wrap(y, areaHeight)

    return x, y
end

function bullets.update(b, timeStep, areaWidth, areaHeight)
    local alive = true

    local x, y = utility.polarToCartesian(b.angle, b.dist,
      b.startX, b.startY)
    particle.setPos(b.effect, x, y, areaWidth, areaHeight)

    particle.update(b.effect, timeStep)

    if b.dist < b.maxDist then
        b.dist = b.dist + b.speed
    else
        if particle.isEmpty(b.effect) then
            alive = false
        else
            b.dist = b.maxDist
            particle.stop(b.effect)
        end
    end

    return alive
end

function bullets.isFading(b)
    return b.dist == b.maxDist
end

function bullets.draw(b, areaWidth, areaHeight)
    local x, y = utility.polarToCartesian(b.angle, b.dist,
      b.startX, b.startY)

    if b.dist < b.maxDist then
        gfx.drawWrappedSprite(b.sprite, x, y, areaWidth, areaHeight,
          b.angle)
    end

    particle.draw(b.effect)
end