--[[
    Aaron MacDonald
    December 3, 2010
]]

require('gfx')
require('utility')

particle = {}

local function emitterDimensions(image, data)
    local maxDist = data.maxLife * data.maxSpeed

    local width = maxDist + (image:getWidth() * data.maxSize)
    local height = maxDist + (image:getHeight() * data.maxSize)

    return width, height
end

function particle.createEffect(data)
    local image = gfx.loadImage(data.spriteName)

    local emitter = love.graphics.newParticleSystem(
      image, data.bufferSize)

    emitter:setLifetime(-1)

    emitter:setEmissionRate(data.emissionRate)
    emitter:setSpeed(data.minSpeed, data.maxSpeed)
    emitter:setSize(data.minSize, data.maxSize)

    emitter:setParticleLife(data.minLife, data.maxLife)
    emitter:setSpread(data.spread)

    local startColour = data.startColour
    local endColour = data.endColour

    emitter:setColor(
      startColour[1], startColour[2], startColour[3], startColour[4],
        endColour[1],   endColour[2],   endColour[3],   endColour[4]
    )

    local width, height = emitterDimensions(image, data)

    return { emitter = emitter,
             width = width,
             height = height,
             radius = data.radius }
end

function particle.isActive(effect)
    return effect.emitter:isActive()
end

function particle.isEmpty(effect)
    return effect.emitter:isEmpty()
end

function particle.start(effect)
    effect.emitter:start()
end

function particle.stop(effect)
    effect.emitter:stop()
end

function particle.setAngle(effect, angle)
    effect.emitter:setDirection(angle)
end

function particle.setPos(effect, x, y, areaWidth, areaHeight, exact)
    local rx = x
    local ry = y

    if not exact then
        rx, ry = utility.polarToCartesian(
          math.random() * (2 * math.pi), effect.radius, x, y)
    end

    rx = utility.wrap(rx, areaWidth)
    ry = utility.wrap(ry, areaHeight)

    effect.emitter:setPosition(rx, ry)
end

function particle.update(effect, timeStep)
    effect.emitter:update(timeStep)
end

function particle.draw(effect)
    love.graphics.draw(effect.emitter, 0, 0)
end