--[[
    Aaron MacDonald
    December 3, 2010
]]

require('gfx')
require('particles')
require('utility')

require('bullet')

require('data.particles')

ship = {}

function ship.create(type, x, y, angle)
    local engine = particle.createEffect(
      ENGINE_PARTICLES[type.engineEffectName])
    particle.stop(engine)

    return { type = type,

             sprite = gfx.createSprite(type.spriteName),
             engine = engine,

             x = x,
             y = y,
             dx = 0,
             dy = 0,
             angle = angle or 0,

             gunRecharge = 0 }
end

----------------------------------------------------------------------

local function accelerate(val, dir, accel, maxSpeed)
    local newVal = val

    if (dir == 0) and (val ~= 0) then
        -- Slow to a stop
        local sign = utility.sign(val)
        newVal = math.abs(val) - accel
        if newVal < 0 then
            newVal = 0
        else
            newVal = newVal * sign
        end
    elseif (dir ~= 0) and (val ~= (maxSpeed * dir)) then
        newVal = val + (dir * accel)
        if math.abs(newVal) > math.abs(maxSpeed * dir) then
            newVal = maxSpeed * dir
        end
    end

    return newVal
end

function ship.updateVelocity(sh, up, down, left, right)
    local dx = 0
    local dy = 0

    if left then
        dx = dx - 1
    end
    if right then
        dx = dx + 1
    end

    if up then
        dy = dy - 1
    end
    if down then
        dy = dy + 1
    end

    if (dx ~= 0) or (dy ~= 0) then
        particle.setAngle(sh.engine, math.atan2(-dy, -dx))

        if not particle.isActive(sh.engine) then
            particle.start(sh.engine)
        end
    else
        particle.stop(sh.engine)
    end

    sh.dx = accelerate(sh.dx, dx, sh.type.accel, sh.type.maxSpeed)
    sh.dy = accelerate(sh.dy, dy, sh.type.accel, sh.type.maxSpeed)
end

function ship.turnTowardsTarget(sh, x, y)
    local targetAngle = math.atan2(y - sh.y, x - sh.x)

    if sh.angle ~= targetAngle then
        local angleDiff = math.atan2(
          math.sin(targetAngle - sh.angle),
          math.cos(targetAngle - sh.angle))

        if math.abs(angleDiff) < sh.type.rotSpeed then
            sh.angle = targetAngle
        else
            local sign = utility.sign(angleDiff)
            sh.angle = sh.angle + (sh.type.rotSpeed * sign)
        end
    end
end

function ship.update(sh, areaWidth, areaHeight, timeStep)
    sh.x = utility.wrap(sh.x + sh.dx, areaWidth)
    sh.y = utility.wrap(sh.y + sh.dy, areaHeight)

    particle.setPos(sh.engine, sh.x, sh.y, areaWidth, areaHeight)
    particle.update(sh.engine, timeStep)

    if sh.gunRecharge > 0 then
        sh.gunRecharge = sh.gunRecharge - 1
    end
end

----------------------------------------------------------------------

function ship.canShoot(sh)
    return sh.gunRecharge == 0
end

function ship.shoot(sh, allBullets)
    local result = false

    if ship.canShoot(sh) then
        local b = bullets.create(sh)
        table.insert(allBullets, b)
        sh.gunRecharge = sh.type.gunRecharge

        result = true
    end

    return result
end

----------------------------------------------------------------------

function ship.draw(sh, areaWidth, areaHeight)
    particle.draw(sh.engine)

    gfx.drawWrappedSprite(sh.sprite, sh.x, sh.y,
      areaWidth, areaHeight, sh.angle)
end