--[[
    Aaron MacDonald
    November 26, 2010
]]

local STAR_SPRITE_NAME = 'star'

local STAR_SPRITE

local NUM_LAYERS = 4
local NUM_STARS = 60

local COLOURS = { {255, 191, 191},  -- Red
                  {255, 223, 191},  -- Orange
                  {255, 255, 191},  -- Yellow
                  {191, 255, 191},  -- Green
                  {191, 191, 255},  -- Blue
                  {255, 191, 255} } -- Violet
local COLOUR_SPEED = 0.1

local STARS

local DX
local DY

starfield = {}

function starfield.init()
    local width = love.graphics.getWidth()
    local height = love.graphics.getHeight()

    STAR_SPRITE = gfx.createSprite(STAR_SPRITE_NAME)

    STARS = {}
    for i = 1, NUM_LAYERS do
        local layer = {}
        for j = 1, NUM_STARS * i do
            table.insert(layer,
              {x = math.random() * width, y = math.random() * height,
               colour = colourCycle.createCycle(COLOURS, COLOUR_SPEED,
                math.random(#COLOURS) - 1, math.random())
              }
            )
        end

        STARS[i] = layer
    end

    --

    DX, DY = utility.polarToCartesian(math.random() * (2 * math.pi))
end

function starfield.update()
    for i, layer in ipairs(STARS) do
        for _, star in ipairs(layer) do
            local scale = (#STARS - i + 1) / #STARS
            local dx = DX * scale
            local dy = DY * scale

            star.x = (star.x + dx) % love.graphics.getWidth()
            if star.x < 0 then
                star.x = star.x + love.graphics.getWidth()
            end

            star.y = (star.y + dy) % love.graphics.getHeight()
            if star.y < 0 then
                star.y = star.y + love.graphics.getHeight()
            end

            colourCycle.update(star.colour)
        end
    end
end

function starfield.draw()
    for i, layer in ipairs(STARS) do
        local scale = (#STARS - i + 1) / #STARS

        for _, star in ipairs(layer) do
            colourCycle.setColour(star.colour)

            gfx.drawWrappedSprite(STAR_SPRITE, star.x, star.y,
              love.graphics.getWidth(), love.graphics.getHeight(),
              nil, scale, scale)

            gfx.setColourWhite()
        end
    end
end